==> profilesx_0.2.tar.gz: profilesx-0.2/control_panel_plugin_src/profilesx-control-panel-plugin.c <==
/*
 *  profilesx
 *  Copyright (C) 2010 Nicolai Hess
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <hildon-cp-plugin/hildon-cp-plugin-interface.h>
#include <hildon/hildon.h>
#include <hildon/hildon-file-chooser-dialog.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <libprofile.h>
#include <keys_nokia.h>
#include <libintl.h>
#include <locale.h>
#include "profilesx-control-panel-plugin.h"
#include "profilesx-profile-waitdialog.h"
#include "profilesx-profile-editor-dialog.h"
	
#define PROFILED_INTERFACE "com.nokia.profiled"
#define PROFILED_PATH "/com/nokia/profiled"
#define PROFILED_SERVICE "com.nokia.profiled"

#define RM_PROFILE_CMD "sudo /usr/bin/profilesx-util -r %s"
#define ADD_PROFILE_CMD "sudo /usr/bin/profilesx-util -a %s"
#define CUSTOM_PROFILE_INI "/etc/profiled/80.profilesx.ini"


gboolean is_predefined_profile(const gchar* profile_name)
{
  return 
    g_strcmp0(profile_name, "silent")==0 ||
    g_strcmp0(profile_name, "general")==0;
}

gchar* value_for_key(profileval_t* values, const gchar* key) 
{ 
  while(values && values->pv_key) 
  { 
    if(g_strcmp0(values->pv_key, key)==0) 
      return values->pv_val; 
    values = values++; 
  } 
  return NULL; 
} 


gchar* get_selected_profile_name(user_data_t* data)
{
  GtkTreeIter iter;
  hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(data->profiles_selector),
				     0,
				     &iter);
  gchar* list_value;
  gtk_tree_model_get(GTK_TREE_MODEL(data->profiles_list_store),
		     &iter,1, &list_value,-1);

  return list_value;
}
void
edit_profile(GtkWidget* button, user_data_t *data)
{
  g_return_if_fail(data);
  gchar* profile_name = get_selected_profile_name(data);
  show_editor_dialog(profile_name, data->main_dialog, data->sound_player);
  g_free(profile_name);
}

gboolean 
is_profile_name_valid(const gchar* profile_name)
{
  return !profile_has_profile(profile_name);
}

void
fill_profiles_list_store(GtkListStore* list_store)
{
  GtkTreeIter iter;
  gchar** profileList = profile_get_profiles();
  gtk_list_store_clear(list_store);
  if(profileList)
  {
    gchar** profilePtr;
    for(profilePtr = profileList;
	*profilePtr;
	profilePtr++)
    {
      gtk_list_store_append(list_store, &iter);
      gtk_list_store_set(list_store, &iter, 
			 0, get_profile_display_name(*profilePtr), 
			 1, *profilePtr, 
			 -1);
    }
  }
  profile_free_profiles(profileList);
}


gchar*
ask_user_for_profile_name(user_data_t* data)
{
  GtkWidget* entry = hildon_entry_new(HILDON_SIZE_AUTO);

  GtkWidget* dialog = gtk_dialog_new_with_buttons(dgettext("osso-profiles", "profi_ti_menu_plugin"),
						  NULL,
						  GTK_DIALOG_MODAL,
						  dgettext("hildon-libs", "wdgt_bd_done"),
						  GTK_RESPONSE_OK,
						  NULL);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), entry);
  gtk_widget_show_all(dialog);

  int ret = 0;
  gboolean name_is_valid = FALSE;



  do{
    ret = gtk_dialog_run(GTK_DIALOG(dialog));
    if(ret == GTK_RESPONSE_OK)
    {
      name_is_valid = is_profile_name_valid(hildon_entry_get_text(HILDON_ENTRY(entry)));
    }
  }while(ret == GTK_RESPONSE_OK && !name_is_valid);
  gchar* new_profile = g_strdup(hildon_entry_get_text(HILDON_ENTRY(entry)));
  gtk_widget_destroy(dialog);
  if(ret == GTK_RESPONSE_OK)
  {
    return new_profile;
  }
  else
  {
    return NULL;
  }
}

void
new_profile(GtkButton* button, user_data_t* data)
{
  gchar* new_profile_name = ask_user_for_profile_name(data);
  if(new_profile_name!=NULL)
  {
    gchar* cmd = g_strdup_printf(ADD_PROFILE_CMD, new_profile_name);
    gboolean ret = g_spawn_command_line_sync(cmd,
					     NULL,
					     NULL,
					     NULL,
					     NULL);
    g_free(cmd);
    if(ret)
    {
      show_progress_bar(new_profile_name, data, TRUE, gtk_widget_get_toplevel(GTK_WIDGET(button)));
      fill_profiles_list_store(data->profiles_list_store);
    }
  }
}

void
activate_general_profile_on_delete_active(const gchar* deleted_profile)
{
  gchar* active_profile = profile_get_profile();
  if(g_strcmp0(active_profile, deleted_profile)==0)
  {
    profile_set_profile("general");
  }
  g_free(active_profile);
}

void
delete_profile(GtkButton* button, user_data_t* data)
{
  gchar* list_value = get_selected_profile_name(data);
  gchar* cmd = g_strdup_printf(RM_PROFILE_CMD, list_value);
  gboolean ret = g_spawn_command_line_sync(cmd,
					   NULL,
					   NULL,
					   NULL,
					   NULL);
  g_free(cmd);
  if(ret)
  {
    activate_general_profile_on_delete_active(list_value);
    show_progress_bar(g_strdup(list_value), data, FALSE, 
		    gtk_widget_get_toplevel(GTK_WIDGET(button)));
  }
  g_free(list_value);
  fill_profiles_list_store(data->profiles_list_store);
}

void
add_main_dialog_buttons(user_data_t* data)
{
  GtkWidget* content_area;
  GtkWidget* edit_button;
  GtkWidget* new_button;
  data->profile_delete_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_delete"));
  edit_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_edit"));
  new_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_new"));

  g_signal_connect(edit_button, "clicked", G_CALLBACK(edit_profile), data);
  g_signal_connect(new_button, "clicked", G_CALLBACK(new_profile), data);
  g_signal_connect(data->profile_delete_button, "clicked", G_CALLBACK(delete_profile), data);

  content_area = gtk_dialog_get_action_area(GTK_DIALOG(data->main_dialog));
  gtk_box_pack_start(GTK_BOX(content_area),
		     data->profile_delete_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(content_area),
		     edit_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(content_area),
		     new_button, FALSE, FALSE, 0);
}

void 
show_hide_delete_button(HildonTouchSelector* selector, gint column, user_data_t* data)
{
  gchar* profile_name = get_selected_profile_name(data);
  if(is_predefined_profile(profile_name))
  {
    gtk_widget_hide(data->profile_delete_button);
  }
  else
  {
    gtk_widget_show(data->profile_delete_button);
  }
  g_free(profile_name);
}

void
add_main_dialog_profile_list(user_data_t* data)
{
  GtkCellRenderer* renderer;
  HildonTouchSelectorColumn* column;
  data->profiles_list_store = gtk_list_store_new(2,
						 G_TYPE_STRING,
						 G_TYPE_STRING);

  fill_profiles_list_store(data->profiles_list_store);

  data->profiles_selector = hildon_touch_selector_new();
  renderer = gtk_cell_renderer_text_new();

  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(data->profiles_selector),
					       GTK_TREE_MODEL(data->profiles_list_store),
					       renderer,
					       "text", 0, NULL);
  g_object_set(G_OBJECT(column),
	       "text-column",
	       0,
	       NULL);
  g_signal_connect(data->profiles_selector, "changed", G_CALLBACK(show_hide_delete_button), data);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(data->main_dialog)->vbox), data->profiles_selector);
}

void create_main_dialog(user_data_t* app_data, GtkWindow* window)
{
  app_data->main_dialog = gtk_dialog_new_with_buttons(dgettext("osso-profiles", "profi_ti_cpa_profiles"),
						      GTK_WINDOW(window),
						      GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
						      dgettext("hildon-libs", "wdgt_bd_done"),GTK_RESPONSE_OK,
						      NULL);
  add_main_dialog_buttons(app_data);
  add_main_dialog_profile_list(app_data);
}

osso_return_t execute(osso_context_t *osso,
		      gpointer data,
		      gboolean user_activated)
{
  gint response;
  setlocale (LC_ALL, "");

  user_data_t* app_data = g_new0(user_data_t, 1);

  gst_init(NULL, NULL);
  app_data->sound_player = gst_element_factory_make("playbin2", "Media Player");
  create_main_dialog(app_data, GTK_WINDOW(data));

  gtk_widget_show_all(app_data->main_dialog);
  show_hide_delete_button(HILDON_TOUCH_SELECTOR(app_data->profiles_selector), 
			  0, 
			  app_data);
  response = gtk_dialog_run(GTK_DIALOG(app_data->main_dialog));
  if(response == GTK_RESPONSE_OK)
  {
    //ignored
  }
  gtk_widget_destroy(GTK_WIDGET(app_data->main_dialog));
  return OSSO_OK;
}

osso_return_t save_state(osso_context_t *osso, gpointer data)
{
  return OSSO_OK;
}

==> profilesx_0.4.tar.gz: profilesx-0.4/control_panel_plugin_src/profilesx-control-panel-plugin.c <==
/*
 *  profilesx
 *  Copyright (C) 2010 Nicolai Hess
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <hildon-cp-plugin/hildon-cp-plugin-interface.h>
#include <hildon/hildon.h>
#include <hildon/hildon-file-chooser-dialog.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <libprofile.h>
#include <keys_nokia.h>
#include <libintl.h>
#include <locale.h>
#include <pulse/pulseaudio.h>
#include "profilesx-control-panel-plugin.h"
#include "profilesx-profile-waitdialog.h"
#include "profilesx-profile-editor-dialog.h"

	
#define PROFILED_INTERFACE "com.nokia.profiled"
#define PROFILED_PATH "/com/nokia/profiled"
#define PROFILED_SERVICE "com.nokia.profiled"

#define RM_PROFILE_CMD "sudo /usr/bin/profilesx-util -r \"%s\""
#define ADD_PROFILE_CMD "sudo /usr/bin/profilesx-util -a \"%s\""
#define CUSTOM_PROFILE_INI "/etc/profiled/80.profilesx.ini"


gboolean is_predefined_profile(const gchar* profile_name)
{
  return 
    g_strcmp0(profile_name, "silent")==0 ||
    g_strcmp0(profile_name, "general")==0;
}

gchar* value_for_key(profileval_t* values, const gchar* key) 
{ 
  while(values && values->pv_key) 
  { 
    if(g_strcmp0(values->pv_key, key)==0) 
      return values->pv_val; 
    values = values++; 
  } 
  return NULL; 
} 


gchar* get_selected_profile_name(user_data_t* data)
{
  GtkTreeIter iter;
  hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(data->profiles_selector),
				     0,
				     &iter);
  gchar* list_value;
  gtk_tree_model_get(GTK_TREE_MODEL(data->profiles_list_store),
		     &iter,1, &list_value,-1);

  return list_value;
}
void
edit_profile(GtkWidget* button, user_data_t *data)
{
  g_return_if_fail(data);
  gchar* profile_name = get_selected_profile_name(data);
  show_editor_dialog(profile_name, data->main_dialog, data->sound_player);
  g_free(profile_name);
}

gboolean 
is_profile_name_valid(const gchar* profile_name)
{
  return !profile_has_profile(profile_name);
}

void
fill_profiles_list_store(GtkListStore* list_store)
{
  GtkTreeIter iter;
  gchar** profileList = profile_get_profiles();
  gtk_list_store_clear(list_store);
  if(profileList)
  {
    gchar** profilePtr;
    for(profilePtr = profileList;
	*profilePtr;
	profilePtr++)
    {
      gtk_list_store_append(list_store, &iter);
      gtk_list_store_set(list_store, &iter, 
			 0, get_profile_display_name(*profilePtr), 
			 1, *profilePtr, 
			 -1);
    }
  }
  profile_free_profiles(profileList);
}


gchar*
ask_user_for_profile_name(user_data_t* data)
{
  GtkWidget* entry = hildon_entry_new(HILDON_SIZE_AUTO);

  GtkWidget* dialog = gtk_dialog_new_with_buttons(dgettext("osso-profiles", "profi_ti_menu_plugin"),
						  NULL,
						  GTK_DIALOG_MODAL,
						  dgettext("hildon-libs", "wdgt_bd_done"),
						  GTK_RESPONSE_OK,
						  NULL);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), entry);
  gtk_widget_show_all(dialog);

  int ret = 0;
  gboolean name_is_valid = FALSE;



  do{
    ret = gtk_dialog_run(GTK_DIALOG(dialog));
    if(ret == GTK_RESPONSE_OK)
    {
      name_is_valid = is_profile_name_valid(hildon_entry_get_text(HILDON_ENTRY(entry)));
    }
  }while(ret == GTK_RESPONSE_OK && !name_is_valid);
  gchar* new_profile = g_strdup(hildon_entry_get_text(HILDON_ENTRY(entry)));
  gtk_widget_destroy(dialog);
  if(ret == GTK_RESPONSE_OK)
  {
    return new_profile;
  }
  else
  {
    return NULL;
  }
}

void
new_profile(GtkButton* button, user_data_t* data)
{
  gchar* new_profile_name = ask_user_for_profile_name(data);
  if(new_profile_name!=NULL)
  {
    gchar* cmd = g_strdup_printf(ADD_PROFILE_CMD, new_profile_name);
    gboolean ret = g_spawn_command_line_sync(cmd,
					     NULL,
					     NULL,
					     NULL,
					     NULL);
    g_free(cmd);
    if(ret)
    {
      show_progress_bar(new_profile_name, data, TRUE, gtk_widget_get_toplevel(GTK_WIDGET(button)));
      fill_profiles_list_store(data->profiles_list_store);
    }
  }
}

void
activate_general_profile_on_delete_active(const gchar* deleted_profile)
{
  gchar* active_profile = profile_get_profile();
  if(g_strcmp0(active_profile, deleted_profile)==0)
  {
    profile_set_profile("general");
  }
  g_free(active_profile);
}

void
delete_profile(GtkButton* button, user_data_t* data)
{
  gchar* list_value = get_selected_profile_name(data);
  gchar* cmd = g_strdup_printf(RM_PROFILE_CMD, list_value);
  gboolean ret = g_spawn_command_line_sync(cmd,
					   NULL,
					   NULL,
					   NULL,
					   NULL);
  g_free(cmd);
  if(ret)
  {
    activate_general_profile_on_delete_active(list_value);
    show_progress_bar(g_strdup(list_value), data, FALSE, 
		    gtk_widget_get_toplevel(GTK_WIDGET(button)));
  }
  g_free(list_value);
  fill_profiles_list_store(data->profiles_list_store);
}

void
add_main_dialog_buttons(user_data_t* data)
{
  GtkWidget* content_area;
  GtkWidget* edit_button;
  GtkWidget* new_button;
  data->profile_delete_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_delete"));
  edit_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_edit"));
  new_button = gtk_button_new_with_label(dgettext("hildon-libs", "wdgt_bd_new"));

  g_signal_connect(edit_button, "clicked", G_CALLBACK(edit_profile), data);
  g_signal_connect(new_button, "clicked", G_CALLBACK(new_profile), data);
  g_signal_connect(data->profile_delete_button, "clicked", G_CALLBACK(delete_profile), data);

  content_area = gtk_dialog_get_action_area(GTK_DIALOG(data->main_dialog));
  gtk_box_pack_start(GTK_BOX(content_area),
		     data->profile_delete_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(content_area),
		     edit_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(content_area),
		     new_button, FALSE, FALSE, 0);
}

void 
show_hide_delete_button(HildonTouchSelector* selector, gint column, user_data_t* data)
{
  gchar* profile_name = get_selected_profile_name(data);
  if(is_predefined_profile(profile_name))
  {
    gtk_widget_hide(data->profile_delete_button);
  }
  else
  {
    gtk_widget_show(data->profile_delete_button);
  }
  g_free(profile_name);
}

void
add_main_dialog_profile_list(user_data_t* data)
{
  GtkCellRenderer* renderer;
  HildonTouchSelectorColumn* column;
  data->profiles_list_store = gtk_list_store_new(2,
						 G_TYPE_STRING,
						 G_TYPE_STRING);

  fill_profiles_list_store(data->profiles_list_store);

  data->profiles_selector = hildon_touch_selector_new();
  renderer = gtk_cell_renderer_text_new();

  column = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(data->profiles_selector),
					       GTK_TREE_MODEL(data->profiles_list_store),
					       renderer,
					       "text", 0, NULL);
  g_object_set(G_OBJECT(column),
	       "text-column",
	       0,
	       NULL);
  g_signal_connect(data->profiles_selector, "changed", G_CALLBACK(show_hide_delete_button), data);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(data->main_dialog)->vbox), data->profiles_selector);
}

void create_main_dialog(user_data_t* app_data, GtkWindow* window)
{
  app_data->main_dialog = gtk_dialog_new_with_buttons(dgettext("osso-profiles", "profi_ti_cpa_profiles"),
						      GTK_WINDOW(window),
						      GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
						      dgettext("hildon-libs", "wdgt_bd_done"),GTK_RESPONSE_OK,
						      NULL);
  add_main_dialog_buttons(app_data);
  add_main_dialog_profile_list(app_data);
}

void init_sound_player(user_data_t* data)
{
  gst_init(NULL, NULL);
  GstElement* pulsesink = gst_element_factory_make("pulsesink", NULL);
  pa_proplist* proplist = pa_proplist_new();
  pa_proplist_sets(proplist,
		  PA_PROP_EVENT_ID,
		  "ringtone-preview");
  pa_proplist_sets(proplist,
		  "module-stream-restore.id",
		  "x-maemo-applet-profiles");
  g_object_set(G_OBJECT(pulsesink), "proplist", proplist, NULL);
  data->sound_player = gst_element_factory_make("playbin2", "Media Player");
  g_object_set(data->sound_player, "audio-sink", pulsesink);
}

osso_return_t execute(osso_context_t *osso,
		      gpointer data,
		      gboolean user_activated)
{
  gint response;
  setlocale (LC_ALL, "");

  user_data_t* app_data = g_new0(user_data_t, 1);
  init_sound_player(app_data);
  create_main_dialog(app_data, GTK_WINDOW(data));

  gtk_widget_show_all(app_data->main_dialog);
  show_hide_delete_button(HILDON_TOUCH_SELECTOR(app_data->profiles_selector), 
			  0, 
			  app_data);
  response = gtk_dialog_run(GTK_DIALOG(app_data->main_dialog));
  if(response == GTK_RESPONSE_OK)
  {
    //ignored
  }
  gtk_widget_destroy(GTK_WIDGET(app_data->main_dialog));
  
  gst_object_unref(app_data->sound_player);
  return OSSO_OK;
}

osso_return_t save_state(osso_context_t *osso, gpointer data)
{
  return OSSO_OK;
}
